from rocolib.api.Parameterized import Parameterized


class Port(Parameterized):
  """
  Abstract base class for a Port
  """
  def __init__(self, parent, params, name='', **kwargs):
    """
    :param parent: component that is holding this port
    :type parent: component
    :param params: parameters to initialize the Parameterized parameters with
    :type params: dict
    :param name: port name
    :type name: basestring
    :param kwargs: additional arguments to override params
    :type kwargs: dict
    """
    super(Port, self).__init__()
    self.parent = parent
    self.setName(name)

    for key, value in params.items():
      self.addParameter(key, value)

    for key, value in kwargs.items():
      self.setParameter(key, value)


  def inherit(self, parent, component):
    # Override to handle getting inherited to parent.interfaces[name]
    pass

  def prefix(self, prefix=""):
    # Override to handle prefixing
    pass


  def canMate(self, otherPort):
    """
    If _allowableMates is an empty list, then returns if self and otherPort
    are the same class.  Otherwise, return if otherPort is an instance of
    any of _allowableMates

    Override this method for better matching
    :returns: whether this port can mate with another port
    :rtype: boolean
    """
    if len(self._allowableMates) > 0:
      for nextType in self._allowableMates:
        if isinstance(otherPort, nextType):
          return True
      return False
    return self.__class__ == otherPort.__class__

  def setParent(self, newParent):
    self.parent = newParent

  def getParent(self):
    return self.parent

  def setName(self, name):
    self.name = str(name)

  def getName(self):
    return self.name

  def toString(self):
    return str(self.parent) + '.' + self.getName()

  def attachTo(self, toPort, composable, **kwargs):
    pass

  def attachFrom(self, fromPort, composable, **kwargs):
    pass


