from rocolib.api.components import FoldedComponent
from rocolib.api.composables.graph.Face import Rectangle as Shape
from rocolib.api.ports import AnchorPort
from rocolib.utils.dimensions import getDim
from rocolib.utils.transforms import Translate, RotateZ
from rocolib.utils.numsym import dot


class ServoMotor(FoldedComponent):
  def define(self):
    self.addParameter("angle", 0, paramType="angle", minValue=None, maxValue=None)
    self.addParameter("servo", "fs90r", paramType="dimension")
    self.addInterface("horn", AnchorPort(self, self.getGraph(), "h", Translate([0,0,0])))
    self.addFaceInterface("mount", "h")

  def assemble(self):
    s = self.getParameter("servo")
    a = self.getParameter("angle")
    dz = getDim(s, "hornheight")

    self.addFace(Shape("h", 0, 0))
    self.setInterface("horn", AnchorPort(self, self.getGraph(), "h", dot(RotateZ(a), Translate([0,0,dz]))))

if __name__ == "__main__":
    ServoMotor.test()
